import { useEffect, useRef, useState } from "react";
import StarRating from './StarRating';

const tempMovieData = [
  {
    imdbID: "tt1375666",
    Title: "Inception",
    Year: "2010",
    Poster:
      "https://m.media-amazon.com/images/M/MV5BMjAxMzY3NjcxNF5BMl5BanBnXkFtZTcwNTI5OTM0Mw@@._V1_SX300.jpg",
  },
  {
    imdbID: "tt0133093",
    Title: "The Matrix",
    Year: "1999",
    Poster:
      "https://m.media-amazon.com/images/M/MV5BNzQzOTk3OTAtNDQ0Zi00ZTVkLWI0MTEtMDllZjNkYzNjNTc4L2ltYWdlXkEyXkFqcGdeQXVyNjU0OTQ0OTY@._V1_SX300.jpg",
  },
  {
    imdbID: "tt6751668",
    Title: "Parasite",
    Year: "2019",
    Poster:
      "https://m.media-amazon.com/images/M/MV5BYWZjMjk3ZTItODQ2ZC00NTY5LWE0ZDYtZTI3MjcwN2Q5NTVkXkEyXkFqcGdeQXVyODk4OTc3MTY@._V1_SX300.jpg",
  },
];

const tempWatchedData = [
  {
    imdbID: "tt1375666",
    title: "Inception",
    Year: "2010",
    poster:
      "https://m.media-amazon.com/images/M/MV5BMjAxMzY3NjcxNF5BMl5BanBnXkFtZTcwNTI5OTM0Mw@@._V1_SX300.jpg",
    runtime: 148,
    imdbRating: 8.8,
    userRating: 10,
  },
  {
    imdbID: "tt0088763",
    title: "Back to the Future",
    Year: "1985",
    poster:
      "https://m.media-amazon.com/images/M/MV5BZmU0M2Y1OGUtZjIxNi00ZjBkLTg1MjgtOWIyNThiZWIwYjRiXkEyXkFqcGdeQXVyMTQxNzMzNDI@._V1_SX300.jpg",
    runtime: 116,
    imdbRating: 8.5,
    userRating: 9,
  },
];

const average = (arr) =>
  arr.reduce((acc, cur, i, arr) => acc + cur / arr.length, 0);



function NavBar({children}){
  return (
    <nav className="nav-bar">
    {children}
  </nav>
  )
}

function NumResults({movies}){
  return (
   
    <p className="num-results">
    Found <strong>{movies.length}</strong> results
    </p>
   

  )
}

function Search({query,setQuery}){
 
  const inputEle=useRef(null);

  useEffect(function(){

    function callback(e){

      if(document.activeElement==inputEle.current){
        return
      }
      if(e.code=='Enter'){
        inputEle.current.focus();
        setQuery("")
      }
   
  }
   document.addEventListener('keydown',callback);
  return ()=> document.addEventListener("keydown",callback);

 } ,[]

);


  
  return (

    <input
    className="search"
    type="text"
    placeholder="Search movies..."
    value={query}
    onChange={(e) => setQuery(e.target.value)} ref={inputEle}
  />
  )
}

function Logo(){
  return (
    <div className="logo">
      <span role="img">🍿</span>
      <h1>MovieRatingApp</h1>
    </div>
  )
}


function Movie({movie,onSelectMovie}){
  return (

    <li onClick={()=>onSelectMovie(movie.imdbID)}>
    <img src={movie.Poster} alt={`${movie.Title} poster`} />
    <h3>{movie.Title}</h3>
    <div>
      <p>
        <span>🗓</span>
        <span>{movie.Year}</span>
        </p>
    </div>
  </li>
  )
}
function MovieList({movies,onSelectMovie}){

   return (
    <ul className="list list-movies">
    {movies?.map((movie) => (
        <Movie movie={movie} key={movie.imdbID} onSelectMovie={onSelectMovie} />
    ))}
  </ul>
  )
}

function Box({children}){
  const [isOpen, setIsOpen] = useState(true);

  return (

    <div className="box">
    <button
      className="btn-toggle"
      onClick={() => setIsOpen((open) => !open)}
    >
      {isOpen ? "–" : "+"}
    </button>
    {isOpen && children}
  </div>
  )
}

function WatchedMovie({movie,onDeleteWatched}){
  return (
    <li >
    <img src={movie.poster} alt={`${movie.Title} poster`} />
    <h3>{movie.title}</h3>
    <div>
      <p>
        <span>⭐️</span>
        <span>{movie.imdbRating}</span>
      </p>
      <p>
        <span>🌟</span>
        <span>{movie.userRating}</span>
      </p>
      <p>
        <span>⏳</span>
        <span>{movie.runtime} min</span>
      </p>
      <button className="btn-delete" onClick={()=> onDeleteWatched(movie.imdbID)}></button>
    </div>
  </li>
  )
}

function WatchedSummary({watched}){
  const avgImdbRating = average(watched.map((movie) => movie.imdbRating));
  const avgUserRating = average(watched.map((movie) => movie.userRating));
  const avgRuntime = average(watched.map((movie) => movie.runtime));

  return (
    <div className="summary">
    <h2>Movies you watched</h2>
    <div>
      <p>
        <span>#️⃣</span>
        <span>{watched.length} movies</span>
      </p>
      <p>
        <span>⭐️</span>
        <span>{avgImdbRating}</span>
      </p>
      <p>
        <span>🌟</span>
        <span>{avgUserRating}</span>
      </p>
      <p>
        <span>⏳</span>
        <span>{avgRuntime} min</span>
      </p>
    </div>
  </div>
  )
}

function WatchedMovieList({watched,onDeleteWatched}){
  return (
   
    <ul className="list">
    {watched.map((movie) => (
       <WatchedMovie movie={movie} key={movie.imdbID} onDeleteWatched={onDeleteWatched} />
    ))}
  </ul>
  )
}


function Loader(){
  console.log("Loading Movies..Please Wait...");
  return <p className="loader">Loading Movies..Please Wait...</p>
}
function Main({children}){
  const [watched, setWatched] = useState(tempWatchedData);
   const [isOpen2, setIsOpen2] = useState(true);
 
  return (
    <main className="main">
    
    
    {children}
 
    </main>
  )
}

function ErrorMessage({message}){
  return <p className="error"> <span>❌{message}</span></p>
}
const KEY="d9a3e4bc";
export default function App() {
  const[selectedId,setSelectedId]=useState(null);
 const [movies, setMovies] = useState(tempMovieData);
 const [isLoading,setIsLoading]=useState(false);
 const [watched, setWatched] = useState(tempWatchedData);
   const [isOpen2, setIsOpen2] = useState(true);
   const [eror,setError]=useState('');
   const [query, setQuery] = useState("test");
 
 // const [movies, setMovies] = useState();


 function handleSelecteMovie(id){
  setSelectedId(selectedId=>id==selectedId?null:id);
 }
 function handleCloseMovie(){
  setSelectedId(null);
 }

 function handleAddWatched(movie){
  setWatched((watched)=>[...watched,movie]);
 }

 function handleDeleteWatched(id){
  setWatched((watched)=> watched.filter((movie)=>movie.imdbID!=id))
 }

useEffect( function(){

  const controller=new AbortController();
 
  async function fetchMovies(){
    try {
          setIsLoading(true);
          setError('');
          const res=await fetch(`http://www.omdbapi.com/?apikey=${KEY}&s=${query}`,{signal:controller.signal});
          if(!res.ok) throw new Error('Some thing Wrong...');
          const data=await res.json();
          console.log(data);
          if(data.Response === 'False') throw new Error("Movie Not Found")
          setMovies(data.Search);
         
    }catch(err){
      console.log('Error in Back End Call',err);
      setError(err.message);
    }
    finally{
      setIsLoading(false);
      setError('')
    }

    if(query.length<=3){
      setMovies([]);
      setError('');
      return;
    }
    
  }
  fetchMovies()
  return function(){
    controller.abort();
  }
},[query]
)


   return (
    <>
     <NavBar >
         <Logo />
        <Search query={query} setQuery={setQuery} />
       <NumResults movies={movies}/>
    </NavBar>
     <Main>
       <Box>
       { /* isLoading ? <Loader /> :  <MovieList movies={movies} /> */ }
       {isLoading && <Loader />}
       {!isLoading && !eror  && <MovieList movies={movies} onSelectMovie={handleSelecteMovie} />}
       {eror && <ErrorMessage message={eror} />}
       </Box>

       <Box>
        {selectedId ? <MovieDetails selectedId={selectedId} 
            onCloseMovie={handleCloseMovie}
            onAddWatched={handleAddWatched} 
            watched={watched} />:(
          <>
     <WatchedSummary watched={watched} />
      <WatchedMovieList watched={watched} onDeleteWatched={handleDeleteWatched} />
      </>
        )
      }
    </Box>
    
      </Main>

      
    </>
  );
}

function MovieDetails({selectedId,onCloseMovie,onAddWatched,watched}){


  const [movie,setMovie]=useState({});
  const[userRating,setUserRating]=useState("");
  const isWatched=watched.map(movie=>movie.imdbID).includes(selectedId);
  const watchedUserRating=watched.find(movie => movie.imdbID==selectedId) ? userRating:0;

  const {Title:title,Year:year,Poster:poster,Runtime:runtime,imdbRating,Plot:plot,
    Released:released,Actors:actors,Director:director,Genre:genre
  }=movie;


  useEffect(
    function(){
      document.title=`Movie | ${title}`
    },[title]
  )

  function handleAdd(){

    const newWatchedMovie={
      imdbID:selectedId,
      title,
      year,
      poster,
      runtime:Number(runtime.split(" ").at(0)),
      //runtime,
      imdbRating:Number(imdbRating),
      userRating

    }
   
    onAddWatched(newWatchedMovie);
    onCloseMovie();

  }

  const [isLoading,setIsLoading]=useState(false);

  useEffect( function(){
 
    async function getMovieDetails(){
      setIsLoading(true);

 const res=await fetch(`http://www.omdbapi.com/?apikey=${KEY}&i=${selectedId}`);
          if(!res.ok) throw new Error('Some thing Wrong...');
          const data=await res.json();
          console.log(data);
          setMovie(data);
          setIsLoading(false);

    }
    getMovieDetails()
  },[selectedId])

  return  ( 
  <div className="details">

{isLoading ?  <Loader /> : (
  <>
    <header>
    <button className="btn btn-back" onClick={onCloseMovie}>&larr;</button>
   
   
   <img src={poster} alt="" />
  

   <div className="details-overview">
    <h2>{title}</h2>
    <p>
      {released} &bull; {runtime}
    </p>
    <p>{genre}</p>

    <p>
      <span>👍</span>
      {imdbRating} IMDB Rating
    </p>
   </div>
    </header>

    <section>

      <div className="rating">

        {!isWatched ? 
              <>
              <StarRating maxRating={10} size={20} onSetRating={setUserRating}/>

              {userRating > 0 && (
              <button className="btn-add" onClick={handleAdd}>+ Add to List</button>
            )}
            </> : 
      
      <p>You Already Rated This Movie {userRating}</p>}
      </div>
    
      <p>
        <em>{plot}</em>
      </p>
      <p>
        Starting {actors}
      </p>
      <p>
        Directed By {director}
      </p>
    </section>
     
    
    </>
)
    
  }
  </div>
  )
  
  
}
